@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -152,40 +152,6 @@ static void write_safe_address (FILE *fp, char *s)
   }
 }
 
-ADDRESS *mutt_get_address (ENVELOPE *env, char **pfxp)
-{
-  ADDRESS *adr;
-  char *pfx = NULL;
-
-  if (mutt_addr_is_user (env->from))
-  {
-    if (env->to && !mutt_is_mail_list (env->to))
-    {
-      pfx = "To";
-      adr = env->to;
-    }
-    else
-    {
-      pfx = "Cc";
-      adr = env->cc;
-    }
-  }
-  else if (env->reply_to && !mutt_is_mail_list (env->reply_to))
-  {
-    pfx = "Reply-To";
-    adr = env->reply_to;
-  }
-  else
-  {
-    adr = env->from;
-    pfx = "From";
-  }
-
-  if (pfxp) *pfxp = pfx;
-
-  return adr;
-}
-
 void mutt_create_alias (ENVELOPE *cur, ADDRESS *iadr)
 {
   ALIAS *new, *t;
@@ -195,7 +161,18 @@ void mutt_create_alias (ENVELOPE *cur, ADDRESS *iadr)
 
   if (cur)
   {
-    adr = mutt_get_address (cur, NULL);
+    if (mutt_addr_is_user (cur->from))
+    {
+      if (cur->to && !mutt_is_mail_list (cur->to))
+	adr = cur->to;
+      else
+	adr = cur->cc;
+    }
+    else if (cur->reply_to && !mutt_is_mail_list (cur->reply_to))
+      adr = cur->reply_to;
+    else
+      adr = cur->from;
+
   }
   else if (iadr)
   {
@@ -326,7 +303,7 @@ int mutt_alias_complete (char *s, size_t buflen)
 {
   ALIAS *a = Aliases;
   ALIAS *a_list = NULL, *a_cur = NULL;
-  char bestname[STRING];
+  char bestname[HUGE_STRING];
   int i;
 
 #define min(a,b)        ((a<b)?a:b)
@@ -420,14 +397,11 @@ int mutt_addr_is_user (ADDRESS *addr)
 
   if (mutt_strcasecmp (addr->mailbox, Username) == 0)
     return 1;
-  if (string_is_address(addr->mailbox, Username, Hostname))
+  if(string_is_address(addr->mailbox, Username, Hostname))
     return 1;
-  if (string_is_address(addr->mailbox, Username, mutt_fqdn(0)))
+  if(string_is_address(addr->mailbox, Username, mutt_fqdn(0)))
     return 1;
-  if (string_is_address(addr->mailbox, Username, mutt_fqdn(1)))
-    return 1;
-
-  if (From && !mutt_strcasecmp (From->mailbox, addr->mailbox))
+  if(string_is_address(addr->mailbox, Username, mutt_fqdn(1)))
     return 1;
 
   if (Alternates.pattern &&
