@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -45,7 +45,6 @@ mutt_copy_hdr (FILE *in, FILE *out, long off_start, long off_end, int flags,
 	       const char *prefix)
 {
   int from = 0;
-  int this_is_from;
   int ignore = 0;
   char buf[STRING]; /* should be long enough to get most fields in one pass */
   char *nl;
@@ -137,19 +136,16 @@ mutt_copy_hdr (FILE *in, FILE *out, long off_start, long off_end, int flags,
     if (nl && buf[0] != ' ' && buf[0] != '\t')
     {
       ignore = 1;
-      this_is_from = 0;
       if (!from && mutt_strncmp ("From ", buf, 5) == 0)
       {
 	if ((flags & CH_FROM) == 0)
 	  continue;
-	this_is_from = from = 1;
+	from = 1;
       }
       else if (buf[0] == '\n' || (buf[0] == '\r' && buf[1] == '\n'))
 	break; /* end of header */
 
-      /* note: CH_FROM takes precedence over header weeding. */
-      if (!((flags & CH_FROM) && (flags & CH_FORCE_FROM) && this_is_from) &&
-	  (flags & CH_WEED) &&
+      if ((flags & CH_WEED) && 
 	  mutt_matches_ignore (buf, Ignore) &&
 	  !mutt_matches_ignore (buf, UnIgnore))
 	continue;
@@ -266,7 +262,6 @@ mutt_copy_hdr (FILE *in, FILE *out, long off_start, long off_end, int flags,
 /* flags
  	CH_DECODE	RFC2047 header decoding
  	CH_FROM		retain the "From " message separator
-        CH_FORCE_FROM	give CH_FROM precedence over CH_WEED
  	CH_MIME		ignore MIME fields
 	CH_NOLEN	don't write Content-Length: and Lines:
  	CH_NONEWLINE	don't output a newline after the header
@@ -400,20 +395,20 @@ static int count_delete_lines (FILE *fp, BODY *b, long *length, size_t datelen)
 }
 
 /* make a copy of a message
- * 
- * fpout	where to write output
- * fpin		where to get input
- * hdr		header of message being copied
- * body		structure of message being copied
- * flags
- * 	M_CM_NOHEADER	don't copy header
- * 	M_CM_PREFIX	quote header and body
- *	M_CM_DECODE	decode message body to text/plain
- *	M_CM_DISPLAY	displaying output to the user
- *	M_CM_UPDATE	update structures in memory after syncing
- *	M_CM_DECODE_PGP	used for decoding PGP messages
- *	M_CM_CHARCONV	perform character set conversion 
- * chflags	flags to mutt_copy_header()
+ 
+   fpout	where to write output
+   fpin		where to get input
+   hdr		header of message being copied
+   body		structure of message being copied
+   flags
+	M_CM_NOHEADER	don't copy header
+ 	M_CM_PREFIX	quote header and body
+ 	M_CM_DECODE	decode message body to text/plain
+ 	M_CM_DISPLAY	displaying output to the user
+	M_CM_UPDATE	update structures in memory after syncing
+	M_CM_DECODE_PGP	used for decoding PGP messages
+        M_CM_CHARCONV	perform character set conversion
+   chflags	flags to mutt_copy_header()
  */
 
 int
@@ -611,7 +606,7 @@ _mutt_append_message (CONTEXT *dest, FILE *fpin, CONTEXT *src, HEADER *hdr,
   if ((msg = mx_open_new_message (dest, hdr, (src->magic == M_MBOX || src->magic == M_MMDF || src->magic == M_KENDRA) ? 0 : M_ADD_FROM)) == NULL)
     return -1;
   if (dest->magic == M_MBOX || dest->magic == M_MMDF || dest->magic == M_KENDRA)
-    chflags |= CH_FROM | CH_FORCE_FROM;
+    chflags |= CH_FROM;
   chflags |= (dest->magic == M_MAILDIR ? CH_NOSTATUS : CH_UPDATE);
   r = _mutt_copy_message (msg->fp, fpin, hdr, body, flags, chflags);
   if (mx_commit_message (msg, dest) != 0)
