@@ -1,6 +1,5 @@
-static const char rcsid[]="$Id: history.c,v 1.5 1998/11/10 22:14:25 roessler Exp $";
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -57,6 +56,9 @@ void mutt_init_history(void)
 {
   history_class_t hclass;
   
+  if (HistSize == OldSize)
+    return;
+  
   for(hclass = HC_FIRST; hclass < HC_LAST; hclass++)
     init_history(&History[hclass]);
 
