@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -58,19 +58,16 @@ enum
   MENU_MAIN,
   MENU_PAGER,
   MENU_POST,
-  MENU_QUERY,
 
-  
 #ifdef _PGPPATH
   MENU_PGP,
 #endif
 
-#ifdef MIXMASTER
-  MENU_MIX,
-#endif
 
 
 
+
+  MENU_QUERY,
   MENU_MAX
 };
 
@@ -105,10 +102,6 @@ extern struct binding_t OpQuery[];
 extern struct binding_t OpPgp[];
 #endif /* _PGPPATH */
 
-#ifdef MIXMASTER
-extern struct binding_t OpMix[];
-#endif
-
 #include "keymap_defs.h"
 
 #endif /* KEYMAP_H */
