@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -36,6 +36,19 @@
 #include "rfc822.h"
 #include "hash.h"
 
+#ifdef ENABLE_NLS
+# include <libintl.h>
+# define _(a) (gettext (a))
+# ifdef gettext_noop
+#  define N_(a) gettext_noop (a)
+# else
+#  define N_(a) (a)
+# endif
+#else
+# define _(a) (a)
+# define N_(a) a
+#endif
+
 #ifdef SUBVERSION
 # define MUTT_VERSION (VERSION SUBVERSION)
 #else  
@@ -51,6 +64,14 @@
 #define INITVAL(x) 
 #endif
 
+#define TRUE 1
+#define FALSE 0
+
+#define HUGE_STRING	5120
+#define LONG_STRING     1024
+#define STRING          256
+#define SHORT_STRING    128
+
 /* flags for mutt_copy_header() */
 #define CH_UPDATE	1      /* update the status and x-status fields? */
 #define CH_WEED		(1<<1) /* weed the headers? */
@@ -66,7 +87,6 @@
 #define CH_TXTPLAIN	(1<<11) /* generate text/plain MIME headers */
 #define CH_NOLEN	(1<<12) /* don't write Content-Length: and Lines: */
 #define CH_WEED_DELIVERED (1<<13) /* weed eventual Delivered-To headers */
-#define CH_FORCE_FROM	(1<<14)	/* give CH_FROM precedence over CH_WEED? */
 
 /* flags for mutt_enter_string() */
 #define  M_ALIAS   1      /* do alias "completion" by calling up the alias-menu */
@@ -122,9 +142,8 @@ typedef enum
 #define M_SENDHOOK	(1<<2)
 #define M_FCCHOOK	(1<<3)
 #define M_SAVEHOOK	(1<<4)
-#define M_CHARSETHOOK	(1<<5)
 #ifdef _PGPPATH
-#define M_PGPHOOK	(1<<6)
+#define M_PGPHOOK	(1<<5)
 #endif
 
 /* tree characters for linearize_tree and print_enriched_string */
@@ -228,14 +247,12 @@ enum
 {
 
 #ifdef _PGPPATH
-  OPT_VERIFYSIG,      /* verify PGP signatures */
-  OPT_PGPTRADITIONAL, /* create old-style PGP messages */
+  OPT_VERIFYSIG, /* verify PGP signatures */
 #endif
 
   OPT_PRINT,
   OPT_INCLUDE,
   OPT_DELETE,
-  OPT_MFUPTO,
   OPT_MIMEFWD,
   OPT_MOVE,
   OPT_COPY,
@@ -244,8 +261,7 @@ enum
   OPT_REPLYTO,
   OPT_ABORT,
   OPT_RECALL,
-  OPT_SUBJECT,
-  OPT_MAX
+  OPT_SUBJECT
 };
 
 /* flags to ci_send_message() */
@@ -257,7 +273,7 @@ enum
 #define SENDBATCH	(1<<5)
 #define SENDMAILX	(1<<6)
 #define SENDKEY		(1<<7)
-#define SENDRESEND	(1<<8)
+#define SENDEDITMSG	(1<<8)
 
 /* boolean vars */
 enum
@@ -277,15 +293,13 @@ enum
   OPTCOLLAPSEUNREAD,
   OPTCONFIRMAPPEND,
   OPTCONFIRMCREATE,
-  OPTDELETEUNTAG,
   OPTEDITHDRS,
-  OPTENCODEFROM,
   OPTFASTREPLY,
   OPTFCCATTACH,
   OPTFOLLOWUPTO,
   OPTFORCENAME,
-  OPTFORWATTACH,
   OPTFORWDECODE,
+  OPTFORWWEEDHEADER,
   OPTFORWQUOTE,
   OPTHDRS,
   OPTHEADER,
@@ -293,8 +307,7 @@ enum
   OPTHIDDENHOST,
   OPTIGNORELISTREPLYTO,
 #ifdef USE_IMAP
-  OPTIMAPLSUB,
-  OPTIMAPPASSIVE,
+    OPTIMAPPASSIVE,
 #endif
   OPTIMPLICITAUTOVIEW,
   OPTMAILCAPSANITIZE,
@@ -316,7 +329,6 @@ enum
   OPTRESOLVE,
   OPTREVALIAS,
   OPTREVNAME,
-  OPTRFC2047PARAMS,
   OPTSAVEADDRESS,
   OPTSAVEEMPTY,
   OPTSAVENAME,
@@ -336,7 +348,6 @@ enum
   OPTWRAP,
   OPTWRAPSEARCH,
   OPTWRITEBCC,		/* write out a bcc header? */
-  OPTXMAILER,
 
   /* PGP options */
   
@@ -346,14 +357,9 @@ enum
   OPTPGPLONGIDS,
   OPTPGPREPLYENCRYPT,
   OPTPGPREPLYSIGN,
-  OPTPGPREPLYSIGNENCRYPTED,
-#if 0
   OPTPGPENCRYPTSELF,
-#endif
-  OPTPGPRETAINABLESIG,
   OPTPGPSTRICTENC,
   OPTFORWDECRYPT,
-  OPTPGPSHOWUNUSABLE,
 #endif
 
   /* pseudo options */
@@ -374,6 +380,7 @@ enum
   OPTFORCEREDRAWPAGER,	/* (pseudo) used to force a redraw in the pager */
   OPTSORTSUBTHREADS,	/* (pseudo) used when $sort_aux changes */
   OPTNEEDRESCORE,	/* (pseudo) set when the `score' command is used */
+  OPTUSEHEADERDATE,	/* (pseudo) used by edit-message */
   OPTATTACHMSG,		/* (pseudo) used by attach-message */
   
 #ifdef _PGPPATH
@@ -463,7 +470,7 @@ typedef struct content
   unsigned int binary : 1; /* long lines, or CR not in CRLF pair */
   unsigned int from : 1;   /* has a line beginning with "From "? */
   unsigned int dot : 1;    /* has a line consisting of a single dot? */
-  unsigned int cr : 1;     /* has CR, even when in a CRLF pair */
+  unsigned int nonasc : 1; /* has unicode characters out of ASCII range */
 } CONTENT;
 
 typedef struct body
@@ -511,18 +518,13 @@ typedef struct body
 				 */
   unsigned int tagged : 1;
   unsigned int deleted : 1;	/* attachment marked for deletion */
-  unsigned int noconv : 1;	/* don't do character set conversion */
-
-#ifdef _PGPPATH
-  unsigned int goodsig : 1;	/* good PGP signature */
-#endif
 
 } BODY;
 
 typedef struct header
 {
 #ifdef _PGPPATH
-  unsigned int pgp : 4;
+  unsigned int pgp : 3;
 #endif
 
   unsigned int mime : 1;    /* has a Mime-Version header? */
@@ -581,13 +583,6 @@ typedef struct header
   struct header *last_sort; /* last message in subthread, for secondary SORT_LAST */
   char *tree;            /* character string to print thread tree */
 
-#ifdef MIXMASTER
-  LIST *chain;
-#endif
-
-#ifdef USE_IMAP
-  void *data;            /* driver-specific data (only used by IMAP */
-#endif
 } HEADER;
 
 #include "mutt_regex.h"
@@ -669,17 +664,15 @@ typedef struct
   int flags;
 } STATE;
 
+
+
 /* flags for the STATE struct */
 #define M_DISPLAY	(1<<0) /* output is displayed to the user */
 
-
-
 #ifdef _PGPPATH
 #define M_VERIFY	(1<<1) /* perform signature verification */
 #endif
 
-
-
 #define M_PENDINGPREFIX (1<<2) /* prefix to write, but character must follow */
 #define M_WEED          (1<<3) /* weed headers even when not in display mode */
 #define M_CHARCONV	(1<<4) /* Do character set conversions */
@@ -693,5 +686,4 @@ void state_prefix_putc(char, STATE *);
 int  state_printf(STATE *, const char *, ...);
 
 #include "protos.h"
-#include "lib.h"
 #include "globals.h"
