@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -51,9 +51,9 @@
 #define IsAttach(x) (x && (x)->bdy)
 #define IsHeader(x) (x && (x)->hdr)
 
-static const char Not_available_in_this_menu[] = N_("Not available in this menu.");
-static const char Mailbox_is_read_only[] = N_("Mailbox is read-only.");
-static const char Function_not_permitted_in_attach_message_mode[] = N_("Function not permitted in attach-message mode.");
+static const char *Not_available_in_this_menu = N_("Not available in this menu.");
+static const char *Mailbox_is_read_only = N_("Mailbox is read-only.");
+static const char *Function_not_permitted_in_attach_message_mode = N_("Function not permitted in attach-message mode.");
 
 #define CHECK_MODE(x)	if (!(x)) \
 			{ \
@@ -891,13 +891,6 @@ fill_buffer (FILE *f, long *last_pos, long offset, unsigned char *buf,
   return b_read;
 }
 
-static int is_ansi (unsigned char *buf)
-{
-  while (buf && (isdigit(*buf) || *buf == ';'))
-    buf++;
-  return (*buf == 'm');
-}
-
 static int grok_ansi(unsigned char *buf, int pos, ansi_attr *a)
 {
   int x = pos;
@@ -1132,7 +1125,7 @@ display_line (FILE *f, long *last_pos, struct line_t **lineInfo, int n,
       c = buf[cnt];
     }
 
-    if (*buf_ptr == '\033' && *(buf_ptr + 1) && *(buf_ptr + 1) == '[' && is_ansi (buf_ptr+2))
+    if (*buf_ptr == '\033' && *(buf_ptr + 1) && *(buf_ptr + 1) == '[')
     {
       cnt = grok_ansi(buf, cnt+3, NULL);
       cnt++;
@@ -1263,7 +1256,7 @@ display_line (FILE *f, long *last_pos, struct line_t **lineInfo, int n,
     }
 
     /* Handle ANSI sequences */
-    if (c == '\033' && buf[ch+1] == '[' && is_ansi (buf+ch+2))
+    if (c == '\033' && buf[ch+1] == '[')
     {
       ch = grok_ansi(buf, ch+2, &a);
       c = buf[ch];
@@ -2095,7 +2088,7 @@ mutt_pager (const char *banner, const char *fname, int flags, pager_t *extra)
 
       case OP_DISPLAY_ADDRESS:
 	CHECK_MODE(IsHeader (extra));
-	mutt_display_address (extra->hdr->env);
+	mutt_display_address (extra->hdr->env->from);
 	break;
 
       case OP_ENTER_COMMAND:
