@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
 #include "mailbox.h"
 #include "mime.h"
 #include "rfc2047.h"
-#include "rfc2231.h"
+
 
 
 #ifdef _PGPPATH
@@ -138,14 +138,11 @@ static PARAMETER *parse_parameters (const char *s)
   const char *p;
   size_t i;
 
-  dprint (2, (debugfile, "parse_parameters: `%s'\n", s));
-  
   while (*s)
   {
     if ((p = strpbrk (s, "=;")) == NULL)
     {
       dprint(1, (debugfile, "parse_parameters: malformed parameter: %s\n", s));
-      rfc2231_decode_parameters (&head);
       return (head); /* just bail out now */
     }
 
@@ -195,8 +192,6 @@ static PARAMETER *parse_parameters (const char *s)
 
       new->value = safe_strdup (buffer);
 
-      dprint (2, (debugfile, "parse_parameter: `%s' = `%s'\n", new->attribute, new->value));
-      
       /* Add this parameter to the list */
       if (head)
       {
@@ -226,7 +221,6 @@ static PARAMETER *parse_parameters (const char *s)
     while (*s == ';'); /* skip empty parameters */
   }    
 
-  rfc2231_decode_parameters (&head);
   return (head);
 }
 
@@ -316,13 +310,6 @@ void mutt_parse_content_type (char *s, BODY *ct)
       ct->subtype = safe_strdup ("x-unknown");
   }
 
-  /* Default character set for text types. */
-  if (ct->type == TYPETEXT)
-  {
-    if (!(pc = mutt_get_parameter ("charset", ct->parameter)))
-      mutt_set_parameter ("charset", "us-ascii", &ct->parameter);
-  }
-
 }
 
 static void parse_content_disposition (char *s, BODY *ct)
@@ -476,7 +463,7 @@ BODY *mutt_parse_messageRFC822 (FILE *fp, BODY *parent)
 
   parent->hdr = mutt_new_header ();
   parent->hdr->offset = ftell (fp);
-  parent->hdr->env = mutt_read_rfc822_header (fp, parent->hdr, 0, 0);
+  parent->hdr->env = mutt_read_rfc822_header (fp, parent->hdr, 0);
   msg = parent->hdr->content;
 
   /* ignore the length given in the content-length since it could be wrong
@@ -524,7 +511,11 @@ BODY *mutt_parse_multipart (FILE *fp, const char *boundary, long end_off, int di
   {
     len = mutt_strlen (buffer);
 
-    crlf =  (len > 1 && buffer[len - 2] == '\r') ? 1 : 0;
+    /* take note of the line ending.  I'm assuming that either all endings
+     * will use <CR><LF> or none will.
+     */
+    if (len > 1 && buffer[len - 2] == '\r')
+      crlf = 1;
 
     if (buffer[0] == '-' && buffer[1] == '-' &&
 	mutt_strncmp (buffer + 2, boundary, blen) == 0)
@@ -720,7 +711,9 @@ time_t mutt_parse_date (const char *s, HEADER *h)
 
       case 2: /* year */
 	tm.tm_year = atoi (t);
-	if (tm.tm_year >= 1900)
+        if (tm.tm_year < 70)
+	  tm.tm_year += 100;
+        else if (tm.tm_year >= 1900)
 	  tm.tm_year -= 1900;
 	break;
 
@@ -858,16 +851,11 @@ void mutt_parse_mime_message (CONTEXT *ctx, HEADER *cur)
  *
  * hdr		header structure of current message (optional).
  * 
- * user_hdrs	If set, store user headers.  Used for recall-message and
+ * user_hdrs	If set, store user headers.  Used for edit-message and 
  * 		postpone modes.
  * 
- * weed		If this parameter is set and the user has activated the
- * 		$weed option, honor the header weed list for user headers.
- * 	        Used for recall-message.
- * 
  */
-ENVELOPE *mutt_read_rfc822_header (FILE *f, HEADER *hdr, short user_hdrs,
-				   short weed)
+ENVELOPE *mutt_read_rfc822_header (FILE *f, HEADER *hdr, short user_hdrs)
 {
   ENVELOPE *e = mutt_new_envelope();
   LIST *last = NULL;
@@ -1188,13 +1176,6 @@ ENVELOPE *mutt_read_rfc822_header (FILE *f, HEADER *hdr, short user_hdrs,
      /* Keep track of the user-defined headers */
     if (!matched && user_hdrs)
     {
-      /* restore the original line */
-      line[strlen (line)] = ':';
-
-      if (weed && option (OPTWEED) && mutt_matches_ignore (line, Ignore)
-	  && !mutt_matches_ignore (line, UnIgnore))
-	continue;
-
       if (last)
       {
 	last->next = mutt_new_list ();
@@ -1202,6 +1183,7 @@ ENVELOPE *mutt_read_rfc822_header (FILE *f, HEADER *hdr, short user_hdrs,
       }
       else
 	last = e->userhdrs = mutt_new_list ();
+      line[strlen (line)] = ':';
       last->data = safe_strdup (line);
     }
 
