@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
- * Copyright (C) 1999 Thomas Roessler <roessler@guug.de>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1999-2000Thomas Roessler <roessler@guug.de>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -23,10 +23,6 @@
 #include "mime.h"
 #include "mailbox.h"
 #include "mapping.h"
-#ifdef USE_IMAP
-#include "mx.h"
-#include "imap.h"
-#endif
 
 #include <ctype.h>
 #include <unistd.h>
@@ -58,28 +54,13 @@ int mutt_num_postponed (void)
   struct stat st;
   CONTEXT ctx;
 
-#ifdef USE_IMAP
-  /* LastModify is useless for IMAP */
-  if (Postponed && mx_is_imap (Postponed))
-  {
-    PostCount = imap_mailbox_check (Postponed, 0);
-    PostCount = (PostCount < 0) ? 0 : PostCount;
-    dprint(2, (debugfile,
-      "mutt_num_postponed: %d postponed IMAP messages found.\n", PostCount));
-
-    return PostCount;
-  }
-#endif
-
   if (!Postponed || stat (Postponed, &st) == -1)
   {
      PostCount = 0;
      LastModify = 0;
      return (0);
   } 
-
-
-  if (S_ISDIR (st.st_mode))
+  else if (S_ISDIR (st.st_mode))
   {
     /* if we have a maildir mailbox, we need to stat the "new" dir */
 
@@ -194,28 +175,10 @@ int mutt_get_postponed (CONTEXT *ctx, HEADER *hdr, HEADER **cur, char *fcc, size
   LIST *next;
   char *p;
   int opt_delete;
-#ifdef USE_IMAP
-  char curpath[LONG_STRING];
-  int need_reopen = 0;
-#endif
 
   if (!Postponed)
     return (-1);
 
-#ifdef USE_IMAP
-  /* if we're in an IMAP folder and the postponed folder is also IMAP, we may
-   * need to take steps to avoid opening an additional connection to the same
-   * server. */
-  if ((ctx && ctx->magic == M_IMAP) && mx_is_imap (Postponed))
-  { 
-    strfcpy (curpath, ctx->path, sizeof (curpath));
-    if (imap_select_mailbox (ctx, Postponed) < 0)
-      return -1;
-    need_reopen = 1;
-    PostContext = ctx;
-  }
-  else
-#endif
   if ((PostContext = mx_open_mailbox (Postponed, M_NOSORT, NULL)) == NULL)
   {
     PostCount = 0;
@@ -227,11 +190,6 @@ int mutt_get_postponed (CONTEXT *ctx, HEADER *hdr, HEADER **cur, char *fcc, size
   {
     PostCount = 0;
     mx_close_mailbox (PostContext);
-#ifdef USE_IMAP
-  if (need_reopen)
-    ctx = mx_open_mailbox (curpath, 0, PostContext);
-  else
-#endif
     safe_free ((void **) &PostContext);
     mutt_error _("No postponed messages.");
     return (-1);
@@ -245,25 +203,15 @@ int mutt_get_postponed (CONTEXT *ctx, HEADER *hdr, HEADER **cur, char *fcc, size
   else if ((h = select_msg ()) == NULL)
   {
     mx_close_mailbox (PostContext);
-#ifdef USE_IMAP
-  if (need_reopen)
-    ctx = mx_open_mailbox (curpath, 0, PostContext);
-  else
-#endif
     safe_free ((void **) &PostContext);
     return (-1);
   }
 
-  if (mutt_prepare_template (PostContext, hdr, h, 0) < 0)
+  if (mutt_prepare_edit_message (PostContext, hdr, h) < 0)
   {
-    mx_fastclose_mailbox (PostContext);
-#ifdef USE_IMAP
-    if (need_reopen)
-      ctx = mx_open_mailbox (curpath, 0, NULL);
-    else
-#endif
-    safe_free ((void **) &PostContext);
-    return (-1);
+      mx_fastclose_mailbox (PostContext);
+      safe_free ((void **) &PostContext);
+      return (-1);
   }
 
   /* finished with this message, so delete it. */
@@ -278,11 +226,6 @@ int mutt_get_postponed (CONTEXT *ctx, HEADER *hdr, HEADER **cur, char *fcc, size
   mx_close_mailbox (PostContext);
   set_quadoption (OPT_DELETE, opt_delete);
 
-#ifdef USE_IMAP
-  if (need_reopen)
-    ctx = mx_open_mailbox (curpath, 0, PostContext);
-  else
-#endif
   safe_free ((void **) &PostContext);
 
   for (tmp = hdr->env->userhdrs; tmp; )
@@ -331,12 +274,9 @@ int mutt_get_postponed (CONTEXT *ctx, HEADER *hdr, HEADER **cur, char *fcc, size
 
 
 #ifdef _PGPPATH
-    else if (mutt_strncmp ("Pgp:", tmp->data, 4) == 0 /* this is generated
-						       * by old mutt versions
-						       */
-	     || mutt_strncmp ("X-Mutt-PGP:", tmp->data, 11) == 0)
+    else if (mutt_strncmp ("Pgp:", tmp->data, 4) == 0)
     {
-      hdr->pgp = mutt_parse_pgp_hdr (strchr (tmp->data, ':') + 1, 1);
+      hdr->pgp = mutt_parse_pgp_hdr (tmp->data+4, 1);
        
       /* remove the pgp field */
       next = tmp->next;
@@ -350,29 +290,7 @@ int mutt_get_postponed (CONTEXT *ctx, HEADER *hdr, HEADER **cur, char *fcc, size
     }
 #endif /* _PGPPATH */
 
-#ifdef MIXMASTER
-    else if (mutt_strncmp ("X-Mutt-Mix:", tmp->data, 11) == 0)
-    {
-      char *t;
-      mutt_free_list (&hdr->chain);
-      
-      t = strtok (tmp->data + 11, " \t\n");
-      while (t)
-      {
-	hdr->chain = mutt_add_list (hdr->chain, t);
-	t = strtok (NULL, " \t\n");
-      }
-      
-      next = tmp->next;
-      if (last) 
-	last->next = tmp->next;
-      else
-	hdr->env->userhdrs = tmp->next;
-      tmp->next = NULL;
-      mutt_free_list (&tmp);
-      tmp = next;
-    }
-#endif
+
 
     else
     {
@@ -470,174 +388,78 @@ int mutt_parse_pgp_hdr (char *p, int set_signas)
 }
 #endif /* _PGPPATH */
 
-static void free_body_list (BODY **bp)
-{
-  BODY *c;
-  
-  for (; *bp; *bp = c)
-  {
-    c = (*bp)->next;
-    mutt_free_body (bp);
-  }
-}
 
 
-int mutt_prepare_template (CONTEXT *ctx, HEADER *newhdr, HEADER *hdr,
-			       short weed)
+int mutt_prepare_edit_message (CONTEXT *ctx, HEADER *newhdr, HEADER *hdr)
 {
-  PARAMETER *par, **ppar;
-  MESSAGE *msg;
+  MESSAGE *msg = mx_open_message (ctx, hdr->msgno);
   char file[_POSIX_PATH_MAX];
-  BODY *b;
-  LIST *p, **q;
 
-  FILE *bfp;
-  
-  if ((msg = mx_open_message (ctx, hdr->msgno)) == NULL)
+  if (msg == NULL)
     return (-1);
 
-  /* parse the message header */
   fseek (msg->fp, hdr->offset, 0);
-  newhdr->env = mutt_read_rfc822_header (msg->fp, newhdr, 1, weed);
+  newhdr->env = mutt_read_rfc822_header (msg->fp, newhdr, 1);
   mutt_free_body (&newhdr->content);
   
-  /* weed user-agent, x-mailer - we don't want them here */
-  p = newhdr->env->userhdrs; 
-  q = &newhdr->env->userhdrs;
-  
-  while (p)
+  if (hdr->content->type == TYPEMESSAGE || hdr->content->type == TYPEMULTIPART)
   {
-    if (!strncasecmp (p->data, "x-mailer:", 9) || !strncasecmp (p->data, "user-agent:", 11))
+    BODY *b;
+
+    mutt_parse_part (msg->fp, hdr->content);
+
+    /* Now that we know what was in the other message, convert to the new
+     * message.
+     */
+    newhdr->content = hdr->content->parts;
+    b = hdr->content->parts;
+    while (b != NULL)
     {
-      *q = p->next;
-      p->next = NULL;
-      mutt_free_list (&p);
+      file[0] = '\0';
+      if (b->filename)
+	strfcpy (file, b->filename, sizeof (file));
+      else
+	/* avoid Content-Disposition: header with temporary filename */
+	b->use_disp = 0;
+      mutt_adv_mktemp (file, sizeof(file));
+      if (mutt_save_attachment (msg->fp, b, file, 0, NULL) == -1)
+      {
+	mutt_free_envelope (&newhdr->env);
+	mutt_free_body (&newhdr->content);
+	mx_close_message (&msg);
+	return (-1);
+      }
+      safe_free ((void *) &b->filename);
+      b->filename = safe_strdup (file);
+      b->unlink = 1;
+      mutt_stamp_attachment (b);
+      mutt_free_body (&b->parts);
+      b = b->next;
     }
-    else
-      q = &p->next;
-
-    p = *q;
-  }
-
-  safe_free ((void **) &newhdr->env->message_id);
-  safe_free ((void **) &newhdr->env->mail_followup_to);
-
-  /* make sure we parse the message */
-  mutt_parse_part (msg->fp, hdr->content);
-
-#ifdef _PGPPATH
-  /* decrypt pgp/mime encoded messages */
-  if ((hdr->pgp | PGPENCRYPT) && mutt_is_multipart_encrypted (hdr->content))
-  {
-    newhdr->pgp |= PGPENCRYPT;
-    if (!pgp_valid_passphrase())
-      goto err;
-
-    mutt_message _("Invoking PGP...");
-    if (pgp_decrypt_mime (msg->fp, &bfp, hdr->content, &b) == -1)
-    {
- err:
-      mx_close_message (&msg);
-      mutt_free_envelope (&newhdr->env);
-      return -1;
-    }
-    mutt_clear_error ();
+    hdr->content->parts = NULL;
   }
   else
-#endif
   {
-    /* copy the MIME structure */
-    b = mutt_new_body ();
-    memcpy (b, hdr->content, sizeof (BODY));
-    hdr->content->parts = NULL;
-    
-    /* detach the temporary MIME structure from the message. */
-    b->xtype = safe_strdup (b->xtype);
-    b->subtype = safe_strdup (b->subtype);
-    b->form_name = safe_strdup (b->form_name);
-    b->filename = safe_strdup (b->filename);
-    b->d_filename = safe_strdup (b->d_filename);
-    
-    /* copy parameters */
-    for (par = b->parameter, ppar = &b->parameter; par; ppar = &(*ppar)->next, par = par->next)
-    {
-      *ppar = mutt_new_parameter ();
-      (*ppar)->attribute = safe_strdup (par->attribute);
-      (*ppar)->value = safe_strdup (par->value);
-    }
-    
-    bfp = msg->fp;
-  }
-
-#ifdef _PGPPATH
-  /* 
-   * remove a potential multipart/signed layer - useful when
-   * resending messages 
-   */
-  
-  if (mutt_is_multipart_signed (b))
-  {
-    newhdr->pgp |= PGPSIGN;
-    
-    /* destroy the signature */
-    mutt_free_body (&b->parts->next);
-    b = mutt_remove_multipart (b);
-  }
-#endif
-  
-  /* 
-   * We don't need no primary multipart.
-   * Note: We _do_ preserve messages!
-   * 
-   * XXX - we don't handle multipart/alternative in any 
-   * smart way when sending messages.
-   * 
-   */
-  
-  if (b->type == TYPEMULTIPART)
-    b = mutt_remove_multipart (b);
-  
-  newhdr->content = b;
-  
-  /* create temporary files for all attachments */
-  for (; b; b = b->next)
-  {
-    file[0] = '\0';
-    if (b->filename)
-    {
-      strfcpy (file, b->filename, sizeof (file));
-      b->d_filename = safe_strdup (b->filename);
-    }
-    else
-      /* avoid Content-Disposition: header with temporary filename */
-      b->use_disp = 0;
-
-    mutt_adv_mktemp (file, sizeof(file));
-    if (mutt_save_attachment (bfp, b, file, 0, NULL) == -1)
+    mutt_mktemp (file);
+    if (mutt_save_attachment (msg->fp, hdr->content, file, 0, NULL) == -1)
     {
       mutt_free_envelope (&newhdr->env);
-      free_body_list (&newhdr->content);
-      if (bfp != msg->fp) fclose (bfp);
       mx_close_message (&msg);
-      return -1;
+      return (-1);
     }
+    newhdr->content = mutt_make_file_attach (file);
     
-    safe_free ((void *) &b->filename);
-    b->filename = safe_strdup (file);
-    b->unlink = 1;
-
-    if (mutt_is_text_type (b->type, b->subtype))
-      b->noconv = 1;
-      
-    mutt_stamp_attachment (b);
-
-    mutt_free_body (&b->parts);
-    if (b->hdr) b->hdr->content = NULL; /* avoid dangling pointer */
+    FREE (&newhdr->content->subtype);
+    FREE (&newhdr->content->xtype);
+    
+    newhdr->content->type = hdr->content->type;
+    newhdr->content->xtype = safe_strdup (hdr->content->xtype);
+    newhdr->content->subtype = safe_strdup (hdr->content->subtype);
+    
+    newhdr->content->use_disp = 0;	/* no content-disposition */
+    newhdr->content->unlink = 1;	/* delete when we are done */
   }
 
-  /* that's it. */
-  if (bfp != msg->fp) fclose (bfp);
   mx_close_message (&msg);
-
   return 0;
 }
